@@ -3,20 +3,22 @@ prototype of a file-reader with various converters
 to generate valid shepherd-data for emulation
 
 """
-from typing import NoReturn
-from pathlib import Path
-import logging
-import pickle
-import math
 import errno
+import logging
+import math
 import os
+import pickle  # noqa: S403
+from pathlib import Path
+from typing import Optional
 
 import numpy as np
 import pandas as pd
 from tqdm import trange
 
 from . import Writer
-from .mppt import OptimalTracker, MPPTracker, iv_model
+from .mppt import MPPTracker
+from .mppt import OptimalTracker
+from .mppt import iv_model
 
 
 def get_voc(coeffs: pd.DataFrame):
@@ -30,53 +32,58 @@ def get_isc(coeffs: pd.DataFrame):
 
 
 class Reader:
-    """ container for converters to shepherd-data
+    """container for converters to shepherd-data"""
 
-    """
-
-    samplerate_sps: int = 50
-    sample_interval_ns: int = int(10**9 // samplerate_sps)
-    sample_interval_s: float = 1 / samplerate_sps
-
-    runtime_s: float = None
-    file_size: int = None
-    data_rate: float = None
-
-    _df: pd.DataFrame = None
-
-    logger: logging.Logger = logging.getLogger("SHPData.IVonne.Reader")
+    _logger: logging.Logger = logging.getLogger("SHPData.IVonne.Reader")
 
     def __init__(
-        self, file_path: Path, samplerate_sps: int = None, verbose: bool = True
+        self,
+        file_path: Path,
+        samplerate_sps: Optional[int] = None,
+        verbose: bool = True,
     ):
-        self.logger.setLevel(logging.INFO if verbose else logging.WARNING)
+        self._logger.setLevel(logging.INFO if verbose else logging.WARNING)
 
-        self.file_path = Path(file_path)
+        self.file_path = Path(file_path).resolve()
+        self.samplerate_sps: int = 50
         if samplerate_sps is not None:
             self.samplerate_sps = samplerate_sps
 
+        self.sample_interval_ns: int = int(10**9 // self.samplerate_sps)
+        self.sample_interval_s: float = 1 / self.samplerate_sps
+
+        self.runtime_s: float = 0
+        self.file_size: int = 0
+        self.data_rate: float = 0
+
+        self._df: Optional[pd.DataFrame] = None
+
     def __enter__(self):
         if not self.file_path.exists():
-            raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), self.file_path.name)
+            raise FileNotFoundError(
+                errno.ENOENT, os.strerror(errno.ENOENT), self.file_path.name
+            )
         with open(self.file_path, "rb") as ifr:
-            self._df = pickle.load(ifr)
+            self._df = pickle.load(ifr)  # noqa: S301
         self._refresh_file_stats()
-        self.logger.info(
+        self._logger.info(
             "Reading data from '%s'\n"
-            "\t- runtime = %s s\n"
-            "\t- size = %s KiB\n"
-            "\t- rate = %s KiB/s",
+            "\t- runtime = %d s\n"
+            "\t- size = %.3f KiB\n"
+            "\t- rate = %.3f KiB/s",
             self.file_path,
             self.runtime_s,
-            round(self.file_size / 2**10, 3),
-            round(self.data_rate / 2**10, 3),
+            self.file_size / 2**10,
+            self.data_rate / 2**10,
         )
         return self
 
-    def __exit__(self, *exc):
+    def __exit__(self, *exc):  # type: ignore
         pass
 
-    def _refresh_file_stats(self) -> NoReturn:
+    def _refresh_file_stats(self) -> None:
+        if self._df is None:
+            raise RuntimeError("IVonne Context was not entered - file not open!")
         self.runtime_s = round(self._df.shape[0] / self.samplerate_sps, 3)
         self.file_size = self.file_path.stat().st_size
         self.data_rate = self.file_size / self.runtime_s if self.runtime_s > 0 else 0
@@ -86,8 +93,8 @@ class Reader:
         shp_output: Path,
         v_max: float = 5.0,
         pts_per_curve: int = 1000,
-        duration_s: float = None,
-    ) -> NoReturn:
+        duration_s: Optional[float] = None,
+    ) -> None:
         """Transforms previously recorded parameters to shepherd hdf database with IV curves.
         Shepherd should work with IV 'surfaces', where we have a stream of IV curves
 
@@ -96,8 +103,10 @@ class Reader:
         :param pts_per_curve: Number of sampling points of the prototype curve
         :param duration_s: time to stop in seconds, counted from beginning
         """
+        if self._df is None:
+            raise RuntimeError("IVonne Context was not entered - file not open!")
         if isinstance(duration_s, (float, int)) and self.runtime_s > duration_s:
-            self.logger.info("  -> gets trimmed to %s s", duration_s)
+            self._logger.info("  -> gets trimmed to %f s", duration_s)
             df_elements_n = min(
                 self._df.shape[0], int(duration_s * self.samplerate_sps)
             )
@@ -105,23 +114,24 @@ class Reader:
             df_elements_n = self._df.shape[0]
 
         if shp_output.exists():
-            self.logger.warning("%s already exists, will skip", shp_output.name)
+            self._logger.warning("%s already exists, will skip", shp_output.name)
             return
 
         v_proto = np.linspace(0, v_max, pts_per_curve)
 
-        with Writer(shp_output, datatype="ivcurve", window_samples=pts_per_curve) as sfw:
-
-            sfw.set_window_samples(pts_per_curve)
-            sfw.set_hostname("IVonne")
+        with Writer(
+            shp_output, datatype="ivcurve", window_samples=pts_per_curve
+        ) as sfw:
+            sfw.store_hostname("IVonne")
             curve_interval_us = round(sfw.sample_interval_ns * pts_per_curve / 1000)
             up_factor = self.sample_interval_ns // sfw.sample_interval_ns
             max_elements = math.ceil(sfw.max_elements // up_factor)
             job_iter = trange(0, df_elements_n, max_elements, desc="generate ivcurves")
 
             for idx in job_iter:
-                df_slice = self._df.iloc[idx : idx + max_elements + 1].copy()
-                df_slice.loc[:, "timestamp"] = pd.TimedeltaIndex(
+                idx_top = min(idx + max_elements, df_elements_n)
+                df_slice = self._df.iloc[idx : idx_top + 1].copy()
+                df_slice["timestamp"] = pd.TimedeltaIndex(
                     data=df_slice["time"], unit="s"
                 )
                 df_slice = df_slice.set_index("timestamp")
@@ -149,9 +159,9 @@ class Reader:
         self,
         shp_output: Path,
         v_max: float = 5.0,
-        duration_s: float = None,
-        tracker: MPPTracker = None,
-    ) -> NoReturn:
+        duration_s: Optional[float] = None,
+        tracker: Optional[MPPTracker] = None,
+    ) -> None:
         """Transforms shepherd IV curves to shepherd IV traces.
 
         For the 'buck' and 'buck-boost' modes, shepherd takes voltage and current traces.
@@ -169,8 +179,10 @@ class Reader:
         :param duration_s: time to stop in seconds, counted from beginning
         :param tracker: VOC or OPT
         """
+        if self._df is None:
+            raise RuntimeError("IVonne Context was not entered - file not open!")
         if isinstance(duration_s, (float, int)) and self.runtime_s > duration_s:
-            self.logger.info("  -> gets trimmed to %s s", duration_s)
+            self._logger.info("  -> gets trimmed to %f s", duration_s)
             df_elements_n = min(
                 self._df.shape[0], int(duration_s * self.samplerate_sps)
             )
@@ -178,7 +190,7 @@ class Reader:
             df_elements_n = self._df.shape[0]
 
         if shp_output.exists():
-            self.logger.warning("%s already exists, will skip", shp_output.name)
+            self._logger.warning("%s already exists, will skip", shp_output.name)
             return
 
         if tracker is None:
@@ -187,23 +199,21 @@ class Reader:
             )
 
         with Writer(shp_output, datatype="ivsample") as sfw:
-
-            sfw.set_hostname("IVonne")
+            sfw.store_hostname("IVonne")
             interval_us = round(sfw.sample_interval_ns / 1000)
             up_factor = self.sample_interval_ns // sfw.sample_interval_ns
             max_elements = math.ceil(sfw.max_elements // up_factor)
-            job_iter = trange(
-                0, df_elements_n, max_elements, desc="generate ivsamples"
-            )
+            job_iter = trange(0, df_elements_n, max_elements, desc="generate ivsamples")
 
             for idx in job_iter:
                 # select (max_elements + 1) elements, so upsampling is without gaps
                 # -> drop a sample at the end
-                df_slice = self._df.iloc[idx : idx + max_elements + 1].copy()
+                idx_top = min(idx + max_elements, df_elements_n)
+                df_slice = self._df.iloc[idx : idx_top + 1].copy()
                 df_slice.loc[:, "voc"] = get_voc(df_slice)
                 df_slice.loc[df_slice["voc"] >= v_max, "voc"] = v_max
                 df_slice = tracker.process(df_slice)
-                df_slice.loc[:, "timestamp"] = pd.TimedeltaIndex(
+                df_slice["timestamp"] = pd.TimedeltaIndex(
                     data=df_slice["time"], unit="s"
                 )
                 df_slice = df_slice[["time", "v", "i", "timestamp"]].set_index(
@@ -225,16 +235,18 @@ class Reader:
         self,
         shp_output: Path,
         v_max: float = 5.0,
-        duration_s: float = None,
-    ) -> NoReturn:
+        duration_s: Optional[float] = None,
+    ) -> None:
         """Transforms ivonne-parameters to upsampled version for shepherd
 
         :param shp_output: Path where the resulting hdf file shall be stored
         :param v_max: Maximum voltage supported by shepherd
         :param duration_s: time to stop in seconds, counted from beginning
         """
+        if self._df is None:
+            raise RuntimeError("IVonne Context was not entered - file not open!")
         if isinstance(duration_s, (float, int)) and self.runtime_s > duration_s:
-            self.logger.info("  -> gets trimmed to %s s", duration_s)
+            self._logger.info("  -> gets trimmed to %f s", duration_s)
             df_elements_n = min(
                 self._df.shape[0], int(duration_s * self.samplerate_sps)
             )
@@ -242,12 +254,11 @@ class Reader:
             df_elements_n = self._df.shape[0]
 
         if shp_output.exists():
-            self.logger.warning("%s already exists, will skip", shp_output.name)
+            self._logger.warning("%s already exists, will skip", shp_output.name)
             return
 
         with Writer(shp_output, datatype="isc_voc") as sfw:
-
-            sfw.set_hostname("IVonne")
+            sfw.store_hostname("IVonne")
             interval_us = round(sfw.sample_interval_ns / 1000)
             up_factor = self.sample_interval_ns // sfw.sample_interval_ns
             max_elements = math.ceil(sfw.max_elements // up_factor)
@@ -256,11 +267,12 @@ class Reader:
             for idx in job_iter:
                 # select (max_elements + 1) elements, so upsampling is without gaps
                 # -> drop a sample at the end
-                df_slice = self._df.iloc[idx : idx + max_elements + 1].copy()
+                idx_top = min(idx + max_elements, df_elements_n)
+                df_slice = self._df.iloc[idx : idx_top + 1].copy()
                 df_slice.loc[:, "voc"] = get_voc(df_slice)
                 df_slice.loc[df_slice["voc"] >= v_max, "voc"] = v_max
                 df_slice.loc[:, "isc"] = get_isc(df_slice)
-                df_slice.loc[:, "timestamp"] = pd.TimedeltaIndex(
+                df_slice["timestamp"] = pd.TimedeltaIndex(
                     data=df_slice["time"], unit="s"
                 )
                 df_slice = df_slice[["time", "voc", "isc", "timestamp"]].set_index(
