@@ -9,8 +9,8 @@ from shepherd_data import mppt
 
 @pytest.fixture
 def example_path() -> Path:
-    here = Path(__file__).absolute().parent
-    return here.parent / "src" / "examples"
+    here = Path(__file__).resolve().parent
+    return here.parent / "examples"
 
 
 def test_convert_ivonne(tmp_path: Path, example_path: Path) -> None:
